@@ -1,17 +1,18 @@
 
 module Apply(applyHints, applyHintFile, applyHintFiles) where
 
+import Control.Applicative
+import Data.Monoid
 import HSE.All
 import Hint.All
-import Control.Applicative
-import Control.Arrow
-import Data.List
+import Data.Tuple.Extra
+import Data.Either
+import Data.List.Extra
 import Data.Maybe
-import Data.Monoid
 import Data.Ord
 import Settings
 import Idea
-import Util
+import Prelude
 
 
 -- | Apply hints to a single file, you may have the contents of the file.
@@ -26,34 +27,36 @@ applyHintFile flags s file src = do
 -- | Apply hints to multiple files, allowing cross-file hints to fire.
 applyHintFiles :: ParseFlags -> [Setting] -> [FilePath] -> IO [Idea]
 applyHintFiles flags s files = do
-    (err, ms) <- unzipEither <$> mapM (\file -> parseModuleApply flags s file Nothing) files
+    (err, ms) <- partitionEithers <$> mapM (\file -> parseModuleApply flags s file Nothing) files
     return $ err ++ executeHints s ms
 
 
 -- | Given a way of classifying results, and a 'Hint', apply to a set of modules generating a list of 'Idea's.
 --   The 'Idea' values will be ordered within a file.
-applyHints :: [Classify] -> Hint -> [Module SrcSpanInfo] -> [Idea]
+applyHints :: [Classify] -> Hint -> [(Module SrcSpanInfo, [Comment])] -> [Idea]
 applyHints cls hints_ ms = concat $
     [ map (classify $ cls ++ mapMaybe readPragma (universeBi m)) $
-        order "" (hintModule hints nm m) ++
-        concat [order (fromNamed d) $ decHints d | d <- moduleDecls m]
-    | (nm,m) <- mns
+        order "" (hintModule hints nm m) `merge`
+        concat [order (fromNamed d) $ decHints d | d <- moduleDecls m] `merge`
+        concat [order "" $ hintComment hints c | c <- cs]
+    | (nm,(m,cs)) <- mns
     , let decHints = hintDecl hints nm m -- partially apply
-    , let order n = map (\i -> i{ideaModule=moduleName m, ideaDecl=n}) . sortBy (comparing ideaSpan)] ++
-    [map (classify cls) (hintModules hints mns)]
+    , let order n = map (\i -> i{ideaModule=moduleName m, ideaDecl=n}) . sortBy (comparing ideaSpan)
+    , let merge = mergeBy (comparing ideaSpan)] ++
+    [map (classify cls) (hintModules hints $ map (second fst) mns)]
     where
-        mns = map (scopeCreate &&& id) ms
+        mns = map (scopeCreate . fst &&& id) ms
         hints = (if length ms <= 1 then noModules else id) hints_
-        noModules h = h{hintModules = \_ -> []} `mappend` mempty{hintModule = \a b -> hintModules h [(a,b)]}
+        noModules h = h{hintModules = const []} `mappend` mempty{hintModule = \a b -> hintModules h [(a,b)]}
 
 
 -- | Given a list of settings (a way to classify) and a list of hints, run them over a list of modules.
-executeHints :: [Setting] -> [Module_] -> [Idea]
+executeHints :: [Setting] -> [(Module_, [Comment])] -> [Idea]
 executeHints s = applyHints [x | SettingClassify x <- s] (allHints s)
 
 
 -- | Return either an idea (a parse error) or the module. In IO because might call the C pre processor.
-parseModuleApply :: ParseFlags -> [Setting] -> FilePath -> Maybe String -> IO (Either Idea Module_)
+parseModuleApply :: ParseFlags -> [Setting] -> FilePath -> Maybe String -> IO (Either Idea (Module_, [Comment]))
 parseModuleApply flags s file src = do
     res <- parseModuleEx (parseFlagsAddFixities [x | Infix x <- s] flags) file src
     case res of
