@@ -1,19 +1,22 @@
 {-# LANGUAGE RecordWildCards #-}
+{-# OPTIONS_GHC -fno-warn-incomplete-patterns #-}
 
 module HLint(hlint, Suggestion, suggestionLocation, suggestionSeverity, Severity(..)) where
 
 import Control.Applicative
-import Control.Monad
+import Control.Monad.Extra
 import System.Console.CmdArgs.Verbosity
 import Data.List
 import System.Exit
+import System.IO
+import Prelude
 
 import CmdLine
 import Settings
 import Report
 import Idea
 import Apply
-import Test.Standard
+import Test.All
 import Grep
 import Test.Proof
 import Util
@@ -57,33 +60,33 @@ hlint args = do
         CmdTest{} -> hlintTest cmd >> return []
 
 hlintHSE :: Cmd -> IO ()
-hlintHSE CmdHSE{..} = do
+hlintHSE c@CmdHSE{..} = do
     v <- getVerbosity
     forM_ cmdFiles $ \x -> do
         putStrLn $ "Parse result of " ++ x ++ ":"
-        res <- parseFile x
+        res <- parseFileWithExts (cmdExtensions c) x
         case res of
             x@ParseFailed{} -> print x
             ParseOk m -> case v of
                 Loud -> print m
                 Quiet -> print $ prettyPrint m
-                _ -> print $ fmap (const ()) m
+                _ -> print $ void m
         putStrLn ""
 
 hlintTest :: Cmd -> IO ()
-hlintTest cmd@CmdTest{..} = do
-    if notNull cmdProof then do
+hlintTest cmd@CmdTest{..} =
+    if not $ null cmdProof then do
         files <- cmdHintFiles cmd
         s <- readSettings2 cmdDataDir files []
         let reps = if cmdReports == ["report.html"] then ["report.txt"] else cmdReports
         mapM_ (proof reps s) cmdProof
      else do
-        failed <- test (\args -> do errs <- hlint args; when (length errs > 0) $ exitWith $ ExitFailure 1) cmdDataDir cmdGivenHints
+        failed <- test cmd (\args -> do errs <- hlint args; unless (null errs) $ exitWith $ ExitFailure 1) cmdDataDir cmdGivenHints
         when (failed > 0) exitFailure
 
 hlintGrep :: Cmd -> IO ()
 hlintGrep cmd@CmdGrep{..} = do
-    encoding <- readEncoding cmdEncoding
+    encoding <- if cmdUtf8 then return utf8 else readEncoding cmdEncoding
     let flags = parseFlagsSetExtensions (cmdExtensions cmd) $ defaultParseFlags{cppFlags=cmdCpp cmd, encoding=encoding}
     if null cmdFiles then
         exitWithHelp
@@ -96,9 +99,9 @@ hlintGrep cmd@CmdGrep{..} = do
 
 hlintMain :: Cmd -> IO [Suggestion]
 hlintMain cmd@CmdMain{..} = do
-    encoding <- readEncoding cmdEncoding
+    encoding <- if cmdUtf8 then return utf8 else readEncoding cmdEncoding
     let flags = parseFlagsSetExtensions (cmdExtensions cmd) $ defaultParseFlags{cppFlags=cmdCpp cmd, encoding=encoding}
-    if null cmdFiles && notNull cmdFindHints then do
+    if null cmdFiles && not (null cmdFindHints) then do
         hints <- concatMapM (resolveFile cmd) cmdFindHints
         mapM_ (\x -> putStrLn . fst =<< findSettings2 flags x) hints >> return []
      else if null cmdFiles then
@@ -126,18 +129,25 @@ runHints cmd@CmdMain{..} flags = do
 
     ideas <- if cmdCross
         then applyHintFiles flags settings cmdFiles
-        else concat <$> parallel [listM' =<< applyHintFile flags settings x Nothing | x <- cmdFiles]
+        else concat <$> parallel [evaluateList =<< applyHintFile flags settings x Nothing | x <- cmdFiles]
     let (showideas,hideideas) = partition (\i -> cmdShowAll || ideaSeverity i /= Ignore) ideas
-    showItem <- if cmdColor then showANSI else return show
-    mapM_ (outStrLn . showItem) showideas
-
-    if null showideas then
-        when (cmdReports /= []) $ outStrLn "Skipping writing reports"
-     else
-        forM_ cmdReports $ \x -> do
-            outStrLn $ "Writing report to " ++ x ++ " ..."
-            writeReport cmdDataDir x showideas
-    outStrLn $
-        (let i = length showideas in if i == 0 then "No suggestions" else show i ++ " suggestion" ++ ['s'|i/=1]) ++
-        (let i = length hideideas in if i == 0 then "" else " (" ++ show i ++ " ignored)")
+    if cmdJson
+        then putStrLn . showIdeasJson $ showideas
+        else do
+            usecolour <- cmdUseColour cmd
+            showItem <- if usecolour then showANSI else return show
+            mapM_ (outStrLn . showItem) showideas
+            if null showideas then
+                when (cmdReports /= []) $ outStrLn "Skipping writing reports"
+             else
+                forM_ cmdReports $ \x -> do
+                    outStrLn $ "Writing report to " ++ x ++ " ..."
+                    writeReport cmdDataDir x showideas
+            unless cmdNoSummary $
+                outStrLn $
+                    (let i = length showideas in if i == 0 then "No suggestions" else show i ++ " suggestion" ++ ['s' | i/=1]) ++
+                    (let i = length hideideas in if i == 0 then "" else " (" ++ show i ++ " ignored)")
     return $ map Suggestion showideas
+
+evaluateList :: [a] -> IO [a]
+evaluateList xs = length xs `seq` return xs
