@@ -9,6 +9,7 @@ import Data.Maybe
 import System.FilePath
 import HSE.Type
 import Language.Haskell.Exts.Annotated.Simplify(sQName, sAssoc)
+import Prelude
 
 
 ---------------------------------------------------------------------
@@ -25,36 +26,39 @@ expOp _ = Nothing
 
 moduleDecls :: Module_ -> [Decl_]
 moduleDecls (Module _ _ _ _ xs) = xs
+moduleDecls _ = [] -- XmlPage/XmlHybrid
 
 moduleName :: Module_ -> String
 moduleName (Module _ Nothing _ _ _) = "Main"
 moduleName (Module _ (Just (ModuleHead _ (ModuleName _ x) _ _)) _ _ _) = x
+moduleName _ = "" -- XmlPage/XmlHybrid
 
 moduleImports :: Module_ -> [ImportDecl S]
 moduleImports (Module _ _ _ x _) = x
+moduleImports _ = [] -- XmlPage/XmlHybrid
 
 modulePragmas :: Module_ -> [ModulePragma S]
 modulePragmas (Module _ _ x _ _) = x
+modulePragmas _ = [] -- XmlPage/XmlHybrid
 
 fromModuleName :: ModuleName S -> String
 fromModuleName (ModuleName _ x) = x
 
-isChar :: Exp_ -> Bool
-isChar (Lit _ Char{}) = True
-isChar _ = False
+fromChar :: Exp_ -> Maybe Char
+fromChar (Lit _ (Char _ x _)) = Just x
+fromChar _ = Nothing
 
-fromChar :: Exp_ -> Char
-fromChar (Lit _ (Char _ x _)) = x
+fromPChar :: Pat_ -> Maybe Char
+fromPChar (PLit _ _ (Char _ x _)) = Just x
+fromPChar _ = Nothing
 
-isString :: Exp_ -> Bool
-isString (Lit _ String{}) = True
-isString _ = False
+fromString :: Exp_ -> Maybe String
+fromString (Lit _ (String _ x _)) = Just x
+fromString _ = Nothing
 
-fromString :: Exp_ -> String
-fromString (Lit _ (String _ x _)) = x
-
-isPString (PLit _ String{}) = True; isPString _ = False
-fromPString (PLit _ (String _ x _)) = x
+fromPString :: Pat_ -> Maybe String
+fromPString (PLit _ _ (String _ x _)) =  Just x
+fromPString _ = Nothing
 
 fromParen :: Exp_ -> Exp_
 fromParen (Paren _ x) = fromParen x
@@ -68,7 +72,7 @@ fromTyParen :: Type s -> Type s
 fromTyParen (TyParen _ x) = fromTyParen x
 fromTyParen x = x
 
-fromDeriving :: Deriving s -> [InstHead s]
+fromDeriving :: Deriving s -> [InstRule s]
 fromDeriving (Deriving _ x) = x
 
 -- is* :: Exp_ -> Bool
@@ -97,6 +101,7 @@ isPatTypeSig PatTypeSig{} = True; isPatTypeSig _ = False
 isQuasiQuote QuasiQuote{} = True; isQuasiQuote _ = False
 isSpliceDecl SpliceDecl{} = True; isSpliceDecl _ = False
 isNewType NewType{} = True; isNewType _ = False
+isRecStmt RecStmt{} = True; isRecStmt _ = False
 
 isSection LeftSection{} = True
 isSection RightSection{} = True
@@ -114,6 +119,7 @@ unqual x = x
 fromQual :: QName S -> Name S
 fromQual (Qual _ _ x) = x
 fromQual (UnQual _ x) = x
+fromQual x = error $ "HSE.Util.fromQual, not a name: " ++ prettyPrint x
 
 isSpecial :: QName S -> Bool
 isSpecial Special{} = True; isSpecial _ = False
@@ -134,6 +140,7 @@ dotApp :: Exp_ -> Exp_ -> Exp_
 dotApp x = InfixApp an x (QVarOp an $ UnQual an $ Symbol an ".")
 
 dotApps :: [Exp_] -> Exp_
+dotApps [] = error "HSE.Util.dotApps, does not work on an empty list"
 dotApps [x] = x
 dotApps (x:xs) = dotApp x (dotApps xs)
 
@@ -168,31 +175,23 @@ getEquations x = [x]
 
 
 toFunBind :: Decl s -> Decl s
-toFunBind (PatBind s (PVar _ name) _ bod bind) = FunBind s [Match s name [] bod bind]
+toFunBind (PatBind s (PVar _ name) bod bind) = FunBind s [Match s name [] bod bind]
 toFunBind x = x
 
 
-fromGuardedAlts :: GuardedAlts s -> Rhs s
-fromGuardedAlts (UnGuardedAlt s x) = UnGuardedRhs s x
-fromGuardedAlts (GuardedAlts s xs) = GuardedRhss s [GuardedRhs a b c | GuardedAlt a b c <- xs]
-
-toGuardedAlts :: Rhs s -> GuardedAlts s
-toGuardedAlts (UnGuardedRhs s x) = UnGuardedAlt s x
-toGuardedAlts (GuardedRhss s xs) = GuardedAlts s [GuardedAlt a b c | GuardedRhs a b c <- xs]
-
-
 -- case and if both have branches, nothing else does
 replaceBranches :: Exp s -> ([Exp s], [Exp s] -> Exp s)
 replaceBranches (If s a b c) = ([b,c], \[b,c] -> If s a b c)
 replaceBranches (Case s a bs) = (concatMap f bs, Case s a . g bs)
     where
-        f (Alt _ _ (UnGuardedAlt _ x) _) = [x]
-        f (Alt _ _ (GuardedAlts _ xs) _) = [x | GuardedAlt _ _ x <- xs]
-        g (Alt s1 a (UnGuardedAlt s2 _) b:rest) (x:xs) = Alt s1 a (UnGuardedAlt s2 x) b : g rest xs
-        g (Alt s1 a (GuardedAlts s2 ns) b:rest) xs =
-                Alt s1 a (GuardedAlts s2 [GuardedAlt a b x | (GuardedAlt a b _,x) <- zip ns as]) b : g rest bs
+        f (Alt _ _ (UnGuardedRhs _ x) _) = [x]
+        f (Alt _ _ (GuardedRhss _ xs) _) = [x | GuardedRhs _ _ x <- xs]
+        g (Alt s1 a (UnGuardedRhs s2 _) b:rest) (x:xs) = Alt s1 a (UnGuardedRhs s2 x) b : g rest xs
+        g (Alt s1 a (GuardedRhss s2 ns) b:rest) xs =
+                Alt s1 a (GuardedRhss s2 [GuardedRhs a b x | (GuardedRhs a b _,x) <- zip ns as]) b : g rest bs
             where (as,bs) = splitAt (length ns) xs
         g [] [] = []
+        g _ _ = error "HSE.Util.replaceBranches: internal invariant failed, lists are of differing lengths"
 replaceBranches x = ([], \[] -> x)
 
 
