@@ -19,7 +19,7 @@ foo = a where {a = 1; b = 2; c = 3}; bar = a where {a = 1; b = 2; c = 3} -- ???
 module Hint.Duplicate(duplicateHint) where
 
 import Hint.Type
-import Control.Arrow
+import Data.Tuple.Extra
 import Data.List hiding (find)
 import qualified Data.Map as Map
 
@@ -56,7 +56,7 @@ find _ (Dupe p mp) = (p, 0)
 add :: Ord val => pos -> [val] -> Dupe pos val -> Dupe pos val
 add pos [] d = d
 add pos (v:vs) (Dupe p mp) = Dupe p $ Map.insertWith f v (add pos vs $ Dupe pos Map.empty) mp
-    where f new old = add pos vs old
+    where f new = add pos vs
 
 
 duplicateOrdered :: Ord val => Int -> [[(SrcSpan,val)]] -> [(SrcSpan,SrcSpan,[val])]
