@@ -20,6 +20,8 @@ test = case x of _ | y <- z -> w
 $(fmap return $ dataD (return []) (mkName "Void") [] [] [])
 {-# LANGUAGE RecursiveDo #-} \
 main = mdo x <- y; return y
+{-# LANGUAGE RecursiveDo #-} \
+main = do {rec {x <- return 1}; print x}
 {-# LANGUAGE ImplicitParams, BangPatterns #-} \
 sort :: (?cmp :: a -> a -> Bool) => [a] -> [a] \
 sort !f = undefined
@@ -65,12 +67,11 @@ f :: x -> (x, x); f x = (x, x) --
 -}
 
 
-module Hint.Extensions where
+module Hint.Extensions(extensionsHint) where
 
 import Hint.Type
 import Data.Maybe
-import Data.List
-import Util
+import Data.List.Extra
 
 
 extensionsHint :: ModuHint
@@ -101,13 +102,12 @@ noNewtypeDeriving = ["Read","Show","Data","Typeable","Generic","Generic1"]
 
 
 usedExt :: Extension -> Module_ -> Bool
-usedExt (UnknownExtension "DeriveGeneric") = hasDerive ["Generic","Generic1"]
 usedExt (EnableExtension x) = used x
 usedExt _ = const True
 
 
 used :: KnownExtension -> Module_ -> Bool
-used RecursiveDo = hasS isMDo
+used RecursiveDo = hasS isMDo & hasS isRecStmt
 used ParallelListComp = hasS isParComp
 used FunctionalDependencies = hasT (un :: FunDep S)
 used ImplicitParams = hasT (un :: IPName S)
@@ -122,9 +122,8 @@ used TemplateHaskell = hasT2 (un :: (Bracket S, Splice S)) & hasS f & hasS isSpl
           f TypQuote{} = True
           f _ = False
 used ForeignFunctionInterface = hasT (un :: CallConv S)
-used PatternGuards = hasS f1 & hasS f2
-    where f1 (GuardedRhs _ xs _) = g xs
-          f2 (GuardedAlt _ xs _) = g xs
+used PatternGuards = hasS f
+    where f (GuardedRhs _ xs _) = g xs
           g [] = False
           g [Qualifier{}] = False
           g _ = True
@@ -140,6 +139,7 @@ used DeriveDataTypeable = hasDerive ["Data","Typeable"]
 used DeriveFunctor = hasDerive ["Functor"]
 used DeriveFoldable = hasDerive ["Foldable"]
 used DeriveTraversable = hasDerive ["Traversable"]
+used DeriveGeneric = hasDerive ["Generic","Generic1"]
 used GeneralizedNewtypeDeriving = any (`notElem` noNewtypeDeriving) . fst . derives
 used Arrows = hasS f
     where f Proc{} = True
@@ -172,22 +172,25 @@ derives = concatUnzip . map f . childrenBi
         f (DataInsDecl _ dn _ _ ds) = g dn ds
         f (GDataInsDecl _ dn _ _ _ ds) = g dn ds
         f (DerivDecl _ _ hd) = (xs, xs) -- don't know whether this was on newtype or not
-            where xs = [h hd]
+            where xs = [ir hd]
         f _ = ([], [])
 
         g dn ds = if isNewType dn then (xs,[]) else ([],xs)
-            where xs = maybe [] (map h . fromDeriving) ds
+            where xs = maybe [] (map ir . fromDeriving) ds
 
-        h (IHead _ a _) = prettyPrint $ unqual a
-        h (IHInfix _ _ a _) = prettyPrint $ unqual a
-        h (IHParen _ a) = h a
+        ir (IRule _ _ _ x) = ih x
+        ir (IParen _ x) = ir x
 
+        ih (IHCon _ a) = prettyPrint $ unqual a
+        ih (IHInfix _ _ a) = prettyPrint $ unqual a
+        ih (IHParen _ a) = ih a
+        ih (IHApp _ a _) = ih a
 
 un = undefined
 
 (&) f g x = f x || g x
 
-hasT t x = notNull (universeBi x `asTypeOf` [t])
+hasT t x = not $ null (universeBi x `asTypeOf` [t])
 hasT2 ~(t1,t2) = hasT t1 & hasT t2
 
 hasS :: Biplate x (f S) => (f S -> Bool) -> x -> Bool
