@@ -66,11 +66,12 @@ baz = bar (\x -> (x +)) -- (+)
 -}
 
 
-module Hint.Lambda where
+module Hint.Lambda(lambdaHint) where
 
 import Hint.Util
 import Hint.Type
 import Util
+import Data.List.Extra
 import Data.Maybe
 
 
