@@ -34,13 +34,14 @@ yes = mapM async ds >>= mapM wait >> return () -- mapM async ds >>= mapM_ wait
 -}
 
 
-module Hint.Monad where
+module Hint.Monad(monadHint) where
 
-import Control.Arrow
 import Control.Applicative
+import Data.Tuple.Extra
 import Data.Maybe
 import Data.List
 import Hint.Type
+import Prelude
 
 
 badFuncs = ["mapM","foldM","forM","replicateM","sequence","zipWithM"]
@@ -96,7 +97,7 @@ monadLet xs = if xs == ys then Nothing else Just ys
         vs = concatMap pvars [p | Generator _ p _ <- xs]
         mkLet (Generator _ (view -> PVar_ p) (fromRet -> Just y))
             | p `notElem` vars y, p `notElem` delete p vs
-            = LetStmt an $ BDecls an [PatBind an (toNamed p) Nothing (UnGuardedRhs an y) Nothing]
+            = LetStmt an $ BDecls an [PatBind an (toNamed p) (UnGuardedRhs an y) Nothing]
         mkLet x = x
 
 fromRet (Paren _ x) = fromRet x
