@@ -24,12 +24,11 @@
 -}
 
 
-module Hint.Pragma where
+module Hint.Pragma(pragmaHint) where
 
 import Hint.Type
 import Data.List
 import Data.Maybe
-import Util
 
 
 pragmaHint :: ModuHint
@@ -46,12 +45,12 @@ pragmaIdea xs ys = rawIdea Error "Use better pragmas" (toSrcSpan $ ann $ head xs
 
 
 languageDupes :: [ModulePragma S] -> [Idea]
-languageDupes [] = []
 languageDupes (a@(LanguagePragma _ x):xs) =
     (if nub_ x `neqList` x
         then [pragmaIdea [a] [LanguagePragma an $ nub_ x]]
-        else [pragmaIdea [a,b] [LanguagePragma an (nub_ $ x ++ y)] | b@(LanguagePragma _ y) <- xs, notNull $ intersect_ x y]) ++
+        else [pragmaIdea [a,b] [LanguagePragma an (nub_ $ x ++ y)] | b@(LanguagePragma _ y) <- xs, not $ null $ intersect_ x y]) ++
     languageDupes xs
+languageDupes _ = []
 
 
 -- Given a pragma, can you extract some language features out
