@@ -2,11 +2,10 @@
 
 module Idea(module Idea, Note(..), showNotes, Severity(..)) where
 
+import Data.List.Extra
 import HSE.All
 import Settings
-import Language.Haskell.HsColour.TTY
-import Language.Haskell.HsColour.Colourise
-import Util
+import HsColour
 
 
 -- | An idea suggest by a 'Hint'.
@@ -22,6 +21,27 @@ data Idea = Idea
     }
     deriving (Eq,Ord)
 
+showIdeaJson :: Idea -> String
+showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = wrap . intercalate "," . map mkPair $
+    [("module", show ideaModule)
+    ,("decl", show ideaDecl)
+    ,("severity", show . show $ ideaSeverity)
+    ,("hint", show ideaHint)
+    ,("file", show srcSpanFilename)
+    ,("startLine", show srcSpanStartLine)
+    ,("startColumn", show srcSpanStartColumn)
+    ,("endLine", show srcSpanEndLine)
+    ,("endColumn", show srcSpanEndColumn)
+    ,("from", show ideaFrom)
+    ,("to", maybe "null" show ideaTo)
+    ,("note", show $ map (show . show) ideaNote)
+    ]
+  where
+    mkPair (k, v) = show k ++ ":" ++ v
+    wrap x = "{" ++ x ++ "}"
+
+showIdeasJson :: [Idea] -> String
+showIdeasJson ideas = "[" ++ intercalate "\n," (map showIdeaJson ideas) ++ "]"
 
 instance Show Idea where
     show = showEx id
@@ -29,8 +49,8 @@ instance Show Idea where
 
 showANSI :: IO (Idea -> String)
 showANSI = do
-    prefs <- readColourPrefs
-    return $ showEx (hscolour prefs)
+    f <- hsColourConsole
+    return $ showEx f
 
 showEx :: (String -> String) -> Idea -> String
 showEx tt Idea{..} = unlines $
@@ -46,6 +66,6 @@ showEx tt Idea{..} = unlines $
 
 rawIdea = Idea "" ""
 idea severity hint from to = rawIdea severity hint (toSrcSpan $ ann from) (f from) (Just $ f to) []
-    where f = ltrim . prettyPrint
+    where f = trimStart . prettyPrint
 warn = idea Warning
 err = idea Error
